package Align;

#
# Standard Perl 'OOP'
#
use vars qw($AUTOLOAD);
use Exporter;
use Carp;
use PfamWWWConfig;

#
# We're so OOP, we don't need to export. <grin>
#


#
# Need 'Sequence and SequenceSet' 
#
use Sequence;


#
#Inheritance Package. 
# 
#
@ISA = qw(Exporter);



my %fields = (
	      name  => undef,
	      seq => undef,   # hash of sequence objects on name
              order => undef,  # hash of orders into name.
	      dis_name => undef # hash of names into display names
);

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    
    my $self = {
	_permitted => \%fields,
	%fields, };

    # seq is a hash of objects

    $self->{seq} = {};
    $self->{order} = {};
    $self->{dis_name} = {};

    bless $self, $class;
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || carp "Ok, $self is not an object of mine!";
    my $name = $AUTOLOAD;
    $name =~ s/.*://; #get only the bit we want
    unless (exists $self->{_permitted}->{$name} ) {
	carp "In type $type, can't access $name";
    }
    if (@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name}
    }
}

sub DESTROY {
}

sub addSequence {
    my $self = shift;
    my $seq = shift;
    my $order = shift;

    return $self->add_Sequence($seq,$order);
}

sub add_Sequence {
    my $self = shift;
    my $seq  = shift;
    my $order = shift;
    my $name;

    $name = $seq->name();
    $start = $seq->start();
    $end  = $seq->end(); 


    if( !defined $order ) {
	$order = keys %{$self->{'seq'}};
    }



    $name = sprintf("%s-%d-%d",$name,$start,$end);


    if( $self->{'seq'}->{$name} ) {
	carp("Replacing one sequence [$name]\n");
    }


    $self->{'seq'}->{$name} = $seq;
    $self->{'order'}->{$order} = $name;



}

sub align_sequence_with_HMM {
    my $self = shift;
    
    
}

sub consensus_string {
    my $self = shift;
    my $len;
    my $out;

    $out = "";

    $len = $self->length();

    foreach $count ( 0 .. $len ) {
	$out .= $self->consensus_aa($count);
    }

    return $out;
    
}

sub consensus_aa {
    my $self = shift;
    my $point = shift;
    my ($seq,%hash,$count,$letter);


    foreach $seq ( $self->eachSequence() ) {
	$letter = substr($seq->{'seq'},$point,1);
	($letter =~ /\./) && next;
	# print "Looking at $letter\n";
	$hash{$letter}++;
    }

    $count = -1;
    $letter = '?';

    foreach $key ( keys %hash ) {
	# print "Now at $key $hash{$key}\n";
	if( $hash{$key} > $count ) {
	    $letter = $key;
	    $count = $hash{$key};
	}
    }

    print "returning $letter\n";
    
    return $letter;
    
}

sub column_frequencies {
    my $self = shift;
    my $fh = shift;

    my ($column,$letter,$aa,%hash);
    my @amino_acids=("A","C","D","E","F","G","H","I","K","L","M","N",
		     "P","Q","R","S","T","V","W","Y",".");

    print $fh "Frequencies for each amino acid per column.\n";

    $len = $self->length();  # Length of alignment

    if ($len>999){
	print $fh "Sorry the output format might look a bit odd\n";
    }

    print $fh "A   C   D   E   F   G   H   I   K   L   M   N   P   Q   R   S   T   V   W   Y   .   #column\n";

    foreach $column ( 0 .. $len-1 ) {

	%hash = (); # clear the hash
	foreach $aa (@amino_acids){
	    $hash{$aa} = 0;
	}

	foreach $seq ( $self->eachSequence() ) {
	    $letter = substr($seq->{'seq'},$column,1);
	    $letter =~ tr/a-z/A-Z/;  # Map lower to upper case
	    $letter =~ tr/-_ /./;    # Map gaps to dots
	    $hash{$letter}++;
	}
	foreach $letter (@amino_acids){
	    printf $fh "%-4d",$hash{$letter};
	}
	printf $fh "%-4d\n",$column+1,"\n";
    }
}



sub write_embl {
    my $self = shift;
    my $file = shift;
    my $hash = shift; # hash for embl entries
    my ($seq,$date);

    $date = `date`;

    $hash->{'CC'} .= "Made by Alignment Module on $date\n ";

    foreach $seq ( $self->eachSequence() ) {
	$hash->{'ID'} = $self->get_displayname($seq->get_nse());
	$seq->write_embl($file,$hash);
    }				
    
    
}

sub write_MSF {
    my $self = shift;
    my $file = shift;
    my $msftag;
    my $type;
    my $count = 0;
    my $maxname;
    my $length;
    
    $date = localtime(time);
    $msftag = "MSF";
    $type = "P";
    $maxname = $self->maxnse_length();
    $length  = $self->length();
    $name = $self->name();
    @arr = "";

    if( !defined $name ) {
	$name = "Align";
    }
    
    print $file "PileUp\n";
    print $file sprintf("\n%s   MSF: %d  Type: P  %s  Check: 00 ..\n\n",$name,$self->no_sequences,$date);

    foreach $seq ( $self->eachSequence() ) {
	$name = $self->get_displayname($seq->get_nse());
	$miss = $maxname - length ($name);
	$miss += 2;
	$pad  = " " x $miss;

	print $file sprintf(" Name: %s%sLen:    %d  Check:  %d  Weight:  1.00\n",$name,$pad,$seq->length(),$seq->GCG_checksum());
	
	$hash{$name} = $seq->seq();
	push(@arr,$name);
    }

    #
    # ok - heavy handed, but there you go.
    #
    print $file "\n//\n";

    while( $count < $length ) {
	
	# there is another block to go!
    
	foreach $name  ( @arr ) {
	    print $file sprintf("%20s  ",$name);
	    
	    $tempcount = $count;
	    $index = 0;
	    while( ($tempcount + 10 < $length) && ($index < 5)  ) {
		
		print $file sprintf("%s ",substr($hash{$name},$tempcount,10));
				    
		$tempcount += 10;
		$index++;
	    }

	    #
	    # ok, could be the very last guy ;)
	    #

	    if( $index < 5) {
		
		#
		# space to print! 
		#

		print $file sprintf("%s ",substr($hash{$name},$tempcount));
		$tempcount += 10;
	    }

	    print $file "\n";
	} # end of each sequence

	print $file "\n\n";

	$count = $tempcount;
    }				
}

sub maxname_length {
    my $self = shift;
    my $maxname = (-1);
    my $seq;

    foreach $seq ( $self->eachSequence() ) {
	$len = length $seq->name();

	if( $len > $maxname ) {
	    $maxname = $len;
	}
    }

    return $maxname;
}

sub maxnse_length {
    my $self = shift;
    my $maxname = (-1);
    my $seq;

    foreach $seq ( $self->eachSequence() ) {
	$len = length $seq->get_nse();

	if( $len > $maxname ) {
	    $maxname = $len;
	}
    }

    return $maxname;
}

sub maxdisplayname_length {
    my $self = shift;
    my $maxname = (-1);
    my $seq;

    foreach $seq ( $self->eachSequence() ) {
	$len = length $self->get_displayname($seq->get_nse());

	if( $len > $maxname ) {
	    $maxname = $len;
	}
    }

    return $maxname;
}

sub length {
    my $self = shift;
    my $seq;
    my $length = (-1);
    my $temp;

    foreach $seq ( $self->eachSequence() ) {
	$temp = $seq->length();
	if( $temp > $length ) {
	    $length = $temp;
	}
    }

    return $length;
}

sub is_flush {
    my $self = shift;
    my $seq;
    my $length = (-1);
    my $temp;
    
    foreach $seq ( $self->eachSequence() ) {
	if( $length == (-1) ) {
	    $length = $seq->length();
	    next;
	}

	$temp = $seq->length();

	if( $temp != $length ) {
	    return 0;
	}
    }

    return 1;
}

sub read_fasta {
    my $self = shift;
    my $in = shift;
    my $count = 0;
    my ($start,$end,$name,$seqname,$seq,$seqchar,$tempname,%align);

    while( <$in> ) {
	if( /^>(\S+)/ ) {
	    $tempname = $1;
	    if( defined $name ) {
		# put away last name and sequence 

		$seq = new Sequence;

		if( $name =~ /(\S+)\/(\d+)-(\d+)/ ) {
		    $seqname = $1;
		    $start = $2;
		    $end = $3;
		} else {
		    $seqname=$name;
		    $start = 1;
		    $end = length($align{$name});
		}
		

		$seq->name($seqname);
		$seq->seq($seqchar);
		$seq->start($start);
		$seq->end($end);

		$self->add_Sequence($seq);

		$count++;
	    }
	    $name = $tempname;
	    $seqchar  = "";
	    next;
	}
	s/[^A-Za-z\.\-]//g;
	$seqchar .= $_;

    }
    # put away last name and sequence 
    
    $seq = new Sequence;
    
    if( $name =~ /(\S+)\/(\d+)-(\d+)/ ) {
	$seqname = $1;
	$start = $2;
	$end = $3;
    } else {
	$seqname=$name;
	$start = 1;
	$end = length($align{$name});
    }
    
    
    $seq->name($seqname);
    $seq->seq($seqchar);
    $seq->start($start);
    $seq->end($end);
    
    $self->add_Sequence($seq);
    $count++;

    return $count;
}
	    

sub read_selex {
    my $self = shift;
    my $in = shift;
    my ($start,$end,%align,$name,$seqname,$seq,$count);
    
    # not quite sure 'what' selex format is, but here we go!

    while( <$in> ) {
	#/^#=SQ/ && last;
	/^#=RF/ && last;
    }

    while( <$in> ) {
	/^#/ && next;
	/^\s/ && next;
	/\w/ && last;
    }

    if( eof($in) ) {
	warn("Exhausted file without reading selex format");
	return undef;
    }
    # first line starting with word
    chop;

    $count =0;

    if( !/^(\S+)\s+([A-Za-z\.\-]+)\s*/ ) {
	warn("Ok. I think [$_] does not look like selex format. Not sure...");
	return (-1);
    }

    $name = $1;
    $seq = $2;

    $align{$name} .= $seq;
    $c2name{$count} = $name;
    $count++;

    while( <$in> ) {
	!/^([^\#]\S+)\s+([A-Za-z\.\-]+)\s*/ && next;
	
	$name = $1;
	$seq = $2;

	if( ! defined $align{$name}  ) {
	    $c2name{$count} = $name;
	    $count++;
	}

	$align{$name} .= $seq;
    }

    # ok... now we can make the sequences

    $count = 0;
    foreach $no ( sort { $a <=> $b } keys %c2name ) {
	$name = $c2name{$no};

	$seq = new Sequence;

	if( $name =~ /(\S+)\/(\d+)-(\d+)/ ) {
	    $seqname = $1;
	    $start = $2;
	    $end = $3;
	} else {
	    $seqname=$name;
	    $start = 1;
	    $end = length($align{$name});
	}


	$seq->name($seqname);
	$seq->seq($align{$name});
	$seq->start($start);
	$seq->end($end);

	$self->add_Sequence($seq);

	$count++;
    }

    return $count;
}
	
sub read_mase {
    my $self = shift;
    my $in = shift;
    my $name;
    my $start;
    my $end;
    my $seq;
    my $add;
    my $count = 0;
	
	
    while( <$in> ) {
	/^;/ && next;
	if( /^(\S+)\/(\d+)-(\d+)/ ) {
	    $name = $1;
	    $start = $2;
	    $end = $3;
	} else {
	    s/\s//g;
	    $name = $_;
	    $end = -1;
	}

	$seq = "";

	while( <$in> ) {
	    /^;/ && last;
	    s/[^A-Za-z\.\-]//g;
	    $seq .= $_;
	}

	$add = new Sequence;
	$add->name($name);
	$add->seq($seq);
	$add->start($start);
	$add->end($end);

	if( $end == -1) {
	    $start = 1;
	    $end = length $seq;
	}
	
	$self->add_Sequence($add);

	$count++;
    }

    return $count;
}
	

sub read_Pfam_file {
    my $self = shift;
    my $file = shift;
    my $out;

    if( open(_TEMPFILE,$file) == 0 ) {
	carp("Could not open $file for reading Pfam style alignment");
	return 0;
    }

    $out = read_Pfam($self,\*_TEMPFILE);
    
    close(_TEMPFILE);

    return $out;
}

sub read_Pfam {
    my $self = shift;
    my $in = shift;
    my $name;
    my $start;
    my $end;
    my $seq;
    my $add;
    my $count = 0;

    while( <$in> ) {
	chop;
	/^#/ && next;
	/^\/\// && last;
	if( !/^(\S+)\/(\d+)-(\d+)\s+(\S+)\s*/ ) { 
	    carp "Found a bad line [$_] in the pfam format alignment";
	    next;
	}

	$name = $1;
	$start = $2;
	$end = $3;
	$seq = $4;


	$add = new Sequence;
     
	$add->name($name);
	$add->seq($seq);
	$add->start($start);
	$add->end($end);

	$self->add_Sequence($add);
	
	$count++;
    }

    return $count;
}

sub write_Pfam_link {
    my $self = shift;
    my $link = shift;
    my $out  = shift;
    my $len  = shift;
    my $acc   = shift;
    my ($namestr,$linkstr,$seq,$name,$start,$end);

    if( !defined $len ) {
	$len = 22;
    }

    foreach $seq ( $self->eachSequence() ) {
	$name = $seq->name();
	$disname = $self->get_displayname($seq->get_nse());

	$add = $len - length $disname;
	$place = " " x $add;
	
	
#	print $out "Going to eval \$linkstr = $link\n";
	eval ("\$linkstr = \"$link\"");
	if( defined $acc ) { 
	    print $out sprintf("%s%s%s %s\n",$linkstr,$place,$seq->seq(),$seq->acc());
	} else {
	    print $out sprintf("%s%s%s\n",$linkstr,$place,$seq->seq());
	}
    }
}

sub write_Pfam {
    my $self = shift;
    my $out  = shift;
    my $acc  = shift;
    my ($namestr,$seq);
    my ($maxn);

    $maxn = $self->maxdisplayname_length();

    foreach $seq ( $self->eachSequence() ) {
	$namestr = $self->get_displayname($seq->get_nse());
	$add = $maxn - length($namestr) + 2;
	$namestr .= " " x $add;
	if( $acc == 1) {
	    print $out sprintf("%s  %s %s\n",$namestr,$seq->seq(),$seq->acc());
	} else {
	    print $out sprintf("%s  %s\n",$namestr,$seq->seq());
	}
    }
}

sub write_Pfam_raw {
    my $self  = shift;
    my $acc  = shift;
    my $out  = shift;
    my $type = shift;

    $fh = new FileHandle;
    #$fh->open("gunzip -c $PfamWWWConfig::data_root/$type/$acc.full.gz | grep -v \\#=GS | ");
    $fh->open("gunzip -c $PfamWWWConfig::data_root/$type/$acc.full.gz | ");
    if( ! defined $fh ) {
      &PfamWWWConfig::exit_html("Failed to open $PfamWWWConfig::data_root/$type/$acc.full.gz");
    }

    while ( <$fh> ) {
      printf $out $_;
    }

    close ($fh);
}


sub write_clustalw {
    my $self = shift;
    my $file = shift;
    my ($count,$length,$seq,@seq,$tempcount);

    print $file "CLUSTAL W(1.4) multiple sequence alignment\n\n\n";
    
    $length = $self->length();
    $count = $tempcount = 0;
    @seq = $self->eachSequence();

    while( $count < $length ) {
	foreach $seq ( @seq ) {
	    print $file sprintf("%-22s %s\n",$self->get_displayname($seq->get_nse()),$seq->subseq($count,$count+50));
	}
	print $file "\n\n";
	$count += 50;
    }

    
}

sub write_fasta {
    my $self = shift;
    my $out  = shift;
    my $dostart = shift;
    my ($seq);

    foreach $seq ( $self->eachSequence() ) {
	$seq->write_fasta($out,$self->get_displayname($seq->get_nse()));
    }
}

sub set_displayname {
    my $self = shift;
    my $name = shift;
    my $disname = shift;

    # print "Setting $name to $disname\n";
    $self->{dis_name}->{$name} = $disname;
}

sub get_displayname {
    my $self = shift;
    my $name = shift;

    if( defined $self->{dis_name}->{$name} ) {
	return  $self->{dis_name}->{$name};
    } else {
	return $name;
    }
}

sub set_displayname_flat {
    my $self = shift;

    foreach $seq ( $self->eachSequence() ) {
	$nse = $seq->get_nse();
	$self->set_displayname($nse,$seq->name());
    }
}


sub set_displayname_count {
    my $self= shift;
    my (@arr,$name,$seq,$count);

    foreach $seq ( $self->each_alphabetically() ) {
	$nse = $seq->get_nse();
	if( $name eq ($seq->name()) ) {
	    $temp = sprintf("%s_%s",$name,$count);
	    $self->set_displayname($nse,$temp);
	    $count++;
	} else {
	    $count = 1;
	    $name = $seq->name();
	    $temp = sprintf("%s_%s",$name,$count);
	    $self->set_displayname($nse,$temp);
	    $count++;
	}
    }

}


	
sub each_alphabetically {
    my $self = shift;
    my ($seq,$nse,@arr,%hash,$count);

    foreach $seq ( $self->eachSequence() ) {
	$nse = $seq->get_nse("-","-");
	$hash{$nse} = $seq;
    }


    foreach $nse ( sort alpha_startend keys %hash) {
	push(@arr,$hash{$nse});
    }

    return @arr;

}

    
sub sort_alphabetically {
    my $self = shift;
    my ($seq,$nse,@arr,%hash,$count);


    foreach $seq ( $self->eachSequence() ) {
	$nse = $seq->get_nse("-","-");
	$hash{$nse} = $seq;
    }

    $count = 0;

    %{$self->{'order'}} = (); # reset the hash;

    foreach $nse ( sort alpha_startend keys %hash) {
	$self->{'order'}->{$count} = $nse;

	$count++;
    }

}


sub map_chars {
    my $self = shift;
    my $from = shift;
    my $to   = shift;

    foreach $seq ( $self->eachSequence() ) {
	$temp = $seq->seq();
	$temp =~ s/$from/$to/g;
	$seq->seq($temp);
    }
}

sub uppercase {
    my $self = shift;
    my $seq;

    foreach $seq ( $self->eachSequence() ) {
	$seq->uppercase();
    }
}


sub alpha_startend {
    ($aname,$astart) = split (/-/,$a);
    ($bname,$bstart) = split (/-/,$b);

    if( $aname eq $bname ) {
	return $astart <=> $bstart;
    }
    else { 
	return $aname cmp $bname;
    }

}

    
sub no_sequences {
    my $self = shift;

    return keys %{$self->{'seq'}};
}

sub no_residues {
    my $self = shift;
    my $count = 0;
    my ($key,$temp);

    foreach $key ( keys %{$self->{'seq'}} ) {

	$temp = $self->{'seq'}->{$key}->seq();


	$temp =~ s/[^A-Za-z]//g;
	
	$count += length $temp;
    }

    return $count;
}

sub eachSequence {
    my $self = shift;
    my (@arr,$order);

    foreach $order ( sort { $a <=> $b } keys %{$self->{'order'}} ) {
	# print STDERR sprintf("Putting %s\n", $self->{'order'}->{$order});
	push(@arr,$self->{'seq'}->{$self->{'order'}->{$order}});
    }

    return @arr;
}

sub show_Pfam {
    my $self = shift;
    my $out  = shift;
    my ($seq);

    foreach $seq ( $self->eachSequence() ) {
	print $out sprintf("%s/%d-%d %s\n",$seq->name(),$seq->start(),$seq->end(),$seq->seq());
    }

}

sub chatter_factor {
    my $self = shift;
    my $count = 0;

    
    $no_residues  = $self->no_residues();

    foreach $seq ( $self->eachSequence() ) {
	$str = $seq->seq();
	while ($str =~ /[\.-][A-Za-z]/g ){
	    $count++;
	}
    }
    if ($no_residues){
	$chatter_factor= $count/$no_residues;
    }

    return $chatter_factor;
}


    


1;
__END__

=head1 NAME

Align.pm - Perl Object for alignments (mainly multiple alignments).

=head1 DESCRIPTION

Align handles multiple alignments of sequences. It is very permissive
of types (it wont insist on things being all same length etc): really
it is a SequenceSet explicitly held in memory with a whole series of
built in manipulations and especially file format systems for
read/writing alignments.

Tricky concepts. Like Sequence module, Align expects name,start,end to
be 'unique' in the alignment, and this is the key for the internal hashes.
(don't forget, name,start,end is abreviated nse in the code). However, in
many cases people don't want the name/start-end to be displayed: either 
multiple names in an alignment or names specific to the alignment (ROA1_HUMAN_1,
ROA1_HUMAN_2 etc). These names are called 'displayname', and generally is what
is used to print out the alignment. They default to name/start-end

=head2 Example1

A simple example to get us started.

	#!/usr/local/bin/perl

	use Align;

        # create a new Sequence

	$ali = new Align;

	#
	# read in from a Mul (Pfam) format file
	#

	$ali->read_Pfam_file("myfile.mul");

	#
	# Make a new alignment of guys only > 30 residues
	#

	$al2 = new Align;


	foreach $seq ( $ali->eachSequence() ) {
		if( $seq->end() - $seq->start() > 30 ) {
			$al2->addSequence($seq);
		}
	}

	

	#
	# write it out in MSF format
	#

	$al2->write_MSF(\*STDOUT);


	

=head1 Method Calls


=head2 Creating Alignments

=over 5

=item new

Align::new

Creates a new alignments. Currently only uninitialized sequences
allowed. 

=back

=head2 Accessing member variables.


There are not many member variables which you should
access directly. The internals of the module (a variety of hashes)
are coordinated by the next set of functions. (Accessing Sequences).
The only thing to change set, is name which is the name of the alignment
itself (eg, Kinase).

=over5 

=item name

	$obj->name() or $obj->name($newname) to get/set respectively

=back

=head2 Accessing sequence objects

=over5

=item addSequence

The basic way to add a Sequence to the alignment. The order is
remembered on respective 'adds'

=item eachSequence

Returns an array of Sequence objects. The array has the order inherent
to the alignment. This is going to be the common idiom for using
align's

	foreach $seq ( $align->eachSequence() )

=item each_alphabetically

Like eachSequence but higher overhead: gives an array of Sequence objects
sorted by C<alpha_startend> but does not alter the 'internal' order.

=back

=head2 Miscallaneous alignment attributes

=over5

=item no_sequences

Returns the number of sequences in the alignment

=item no_residues

Returns the B<total> number of residues [A-Za-z] in the alignment

=item length

Returns the B<max> length of sequence in the alignment. Use is_flush
to check that all sequences are the same length

=item is_flush

if all sequences have the same length, returns 1, otherwise returns 0


=back

=head2 I/O

As in all of current I/O files are passed as references to globs, ie,
\*FILEHANDLE. 

=over5

=item read_fasta FILEH

reads a fasta formatted file

=item read_selex FILEH

reads a selex formatted file

=item read_mase FILEH

reads a mase (seaview) formatted file

=item read_Pfam FILEH

reads a Pfam (MUL) formatted file

=item write_Pfam FILEH

writes a Pfam file

=item write_Pfam_raw name FILEH aln_type

efetches and writes a Pfam alignment

=item write_clustalw

writes a clustalw (version 1.4) formatted file

=item write_MSF FILEH

writes a MSF file

=item write_fasta FILEH

writes a fasta file

=item write_Pfam_link FILEH linkstr


Designed for producing HTML pages. If you want to see
quite an interesting way of having very 'configurable' 
output scripts, look at this. 

=item write_embl FILEH ref-hash

writes an EMBL file 

=back

=head2 Display name manipulations

Remember, display names are specific to B<alignments> 
not sequences (this is counterintuitive at first, but
gets better with time).

=over5

=item set_displayname

Sets the sequence with the nse-name key to displayname. To set
a particular sequence therefore it would be

	foreach $seq ( $align->eachSequence() ) {
		$name = $seq->name();
		$name =~ tr/[A-Z]/[a-z]/;
		$align->set_displayname($seq->get_nse(),$name);
		}

=item set_displayname_count

Sets all sequences to be Name_# where the number is the number
of times this Name has been seen on an L<each_alphabetically>
iteration.

=item set_displayname_flat

Sets all sequences to be plain name. Therefore could have mulitple
names

=back


=item uppercase

No current documentation

=item align_sequence_with_HMM

No current documentation

=item consensus_string

No current documentation

=item consensus_aa

No current documentation

=item column_frequencies

Writes out protein alignment column frequencies to a file handle.
Translates all amino acids to upper case, and all gap characters to dots.

Output is of the form

  A   C   D   E   F   G   H   I   K   L   M   N   P   Q   R   S   T   V   W   Y   .   #column
  2   0   0   4   0   1   1   4   1   13  0   0   68  1   0   3   2   6   0   1   2   1   
  0   0   0   2   0   1   0   0   1   3   0   0   1   0   0   0   1   1   0   0   99  2   

where #column is the number of column in the alignment. The numbers
are counts of residues rather than percentages. 

Here is a test script for this function.
  $a = new Align;

  $a->read_Pfam(\*STDIN);

  $a->column_frequencies(\*STDOUT);



Written by Alex Bateman.







=item chatter_factor

Returns a scalar between zero and one.  This scalar is the chatter factor and
is a measure of alignment quality.  The lower the number is the better the
alignment is.

A badly chattered alignment generally is spoilt by a small insertions in
otherwise reasonable blocks of similarity. A partial example is shown below.

ARAG_ECOLI/197-219    LYV.SH..R.M.E.E.I.FA...LS...DA.ITVFKDG
ARAG_ECOLI/453-475    LFA.SS..D.L.P.E.V.LG...VA...DR.IVVMREG
ARTP_ECOLI/195-217    VIV.TH..E.V.E.V.A.RK...TA...SR.VVYMENG
ARTP_HAEIN/196-218    VIV.TH..E.V.N.V.A.QK...VA...TK.VVYMEQG
ATM1_YEAST/627-648    VYI.AH..R.L.R.T...IA...DA...DK.IIVLDNG
BCRA_BACLI/187-209    FIS.SH..I.L.S.E.I.EQ...LV...DH.VGIIHEG
BEXA_HAEIN/175-197    ILV.SH..S.P.S.A.M.KS...YC...DN.AVVLENG
BFR1_SCHPO/363-386    FVT.AY..Q.A.S.EkI.YK...LF...DR.ICVLYAG
BFR1_SCHPO/1077-1100  LCT.IH..Q.P.S.A.V.LF...DQ..fDR.LLLLQKG

The chatter factor is calculated as the number of gap characters followed by
an amino acid, divided by the total number of amino acids in the alignment.


Written by Alex Bateman




