
#
# Perl Module for GIFAlign
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package GIFAlign;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;

use PfamWWWConfig;
use Paging;
use swiss2gif;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' , 'Align' );


sub view_all_figures {

    my $self = shift;
    shift;	    # Why must this be necessary...?
    my $zoom_factor = shift;
    my $verbose = shift;

    my ($disname,$seq,$name);
    my $entry;
    my $number;    
    local *colorsA; # Arrays that hold already assigned colors
    local *colorsB;
    my ($p,$h,$f,$famname);
    my $text_switch;

    if ($verbose) {
      $text_switch = 2;
    }
    else {
      $text_switch = 0;
    }

    # make a new paging module
   
    $famname = $self->name();
    $p = new Paging;
    $p->dir($PfamWWWConfig::tempdir);
    $p->chunk(20);
    $p->maxkbytes(2000); #2 Mbytes in the cache system
    
    $h = &PfamWWWConfig::header("$famname proteins");
    $f = &PfamWWWConfig::footer();

    $p->header("$h <p>All proteins for $famname (cont...)");
    $p->footer("<hr>Next page <a href=\\\"$PfamWWWConfig::image_temp/\$next_file\\\">here</a><p>$f");
    
    $self->set_displayname_count(); # Count to avoid multiple displays
    
    foreach $seq ( $self->eachSequence() ) {
	$name = $seq->name();
	$disname = $self->get_displayname($seq->get_nse());
	$disname=~ /_(\d+)$/;
        $number= $1; 
		   
	if ($number==1) { # Only show protein the first time
	    $entry=&PfamWWWConfig::swisspfam_FH_by_name($name);

	    # was putting in $p instead of undef, but actually doesn't help much

	    &swiss2gif::domain_view($entry,$text_switch,0,*colorsA,*colorsB,undef,$zoom_factor);
        }
	$p->break();
    }
}


1;  # says use was ok
__END__

=head1 NAME

GIFAlign

=head1 DESCRIPTION

Description for B<GIFAlign>

GIFAlign is a very simple subclass of Align which allows the integration
of swiss2gif code (this is for the family view in Pfam web site). 

The philosophy of this subclassing is to allow Align to be still loosely
coupled to the web site whereas GIFAlign is tightly coupled to swiss2gif
(and swiss2gif is tightly coupled to PfamWWWConfig.pm ... )


=head1 AUTHOR

B<Ewan Birney> Email birney@sanger.ac.uk

=over
