
#
# Perl Module for PfamWWWConfig
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package PfamWWWConfig;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use FileHandle;

use Align;

no strict qw( vars );


$file_root = '/seq/www/Pfam';
$data_root = "$file_root/data";

$desc_dir  = "$data_root/desc";
$full_dir  = "$data_root/full";
$seed_dir  = "$data_root/seed";

$site      = 'http://kisac.cgr.ki.se';
$WWW_root  = "$site/Pfam";
$cgibin    = "$site/cgi-bin/Pfam";
$local_root = "/Pfam";

$www_full_aln = "$WWW_root/getalignment_form.html";
$belvu_help   = "$WWW_root/help/belvu_setup.html";
$stable_link  = "$WWW_root/help/othersite.html";

#image stuff

$image_dir = "$file_root/gifs";
$image_link = "$WWW_root/gifs";
$image_temp = "$WWW_root/temp";

# Main names for domain images. Numbers added automatically.

$imageA="colorsA";
$imageB="colors";

# Extra images

$space  ="$image_link/space.gif";
$overlap="$image_link/overlap.gif";
$blank  ="$image_link/blank2.gif";


# stuff for database search

$tempdir  = "$file_root/temp";
$prog     = "$file_root/hmmpfam";
$thr      = "$data_root/pfam.thr";
$db       = "$data_root/Pfam";
$fragdb   = "$data_root/PfamFrag";

# flag only used by the sanger centre to prevent
# cascading complitation around the scripts
# - This sets the script up to use the LSF system

$use_lsf = 0;

# stuff for jalview

$jaljar    = "$WWW_root/jalview.jar";
$mailserver = "$site";
$jalview_url = "http://circinus.ebi.ac.uk:6543/jalview/help.html";


# script names

$getacc    = "$cgibin/getacc";
$swisspfam = "$cgibin/swisspfamget.pl";
$getalign  = "$cgibin/getalignment.pl";
$getpfamb  = "$cgibin/pfambget.pl";
$search    = "$cgibin/nph-search.html";
$text_s    = "$cgibin/querypfam.pl";
$getall    = "$cgibin/getallproteins.pl";
$align2seed = "$cgibin/align2seed.pl";

# contact point 

$contact   =  '<a href="mailto:Erik Sonnhammer@cgr.ki.se">Pfam</a>';
$email     =  'pfam@cgr.ki.se';
$adminname =  'pfam@cgr.ki.se';

#headers and footers

$header_file = "$file_root/cgr_cgi.header";
$footer_file = "$file_root/cgr_cgi.footer";

#URL to link entries to

$swiss_link   = 'http://www.sanger.ac.uk/srs5bin/cgi-bin/wgetz?-e+[pfamseq-id:$name]';
$prosite_link = 'http://www.sanger.ac.uk/srs5bin/cgi-bin/wgetz?-e+[PROSITEDOC-ID:$name]';

%des_hash  = (
              'AC' => 'Accession',
              'DE' => 'Description',
              'AU' => 'Author',
              'AL' => 'Alignment method of seed',
              'BM' => 'Building method of HMM',
              'SE' => 'Source of seed members',
              'GA' => 'Gathering method',
              'NC' => 'Noise   cutoff',
              'TC' => 'Trusted cutoff',
	      'TP' => 'Type',
              'RC' => 'Reference comment',
              'RN' => 'Reference number',
              'RT' => 'Reference title',
              'RA' => 'Reference author',
              'RL' => 'Reference location',
              'DC' => 'Database comment',
              'PI' => 'Previous IDs',
              'CC' => 'Comment'
	      );

sub get_description_hash {
    return \%des_hash;
}


%dbmapper = ( 
		 'PROSITE' =>   ['Expasy http://expasy.hcuge.ch/cgi-bin/get-prodoc-entry?$id',
				'SRS-UK http://www.sanger.ac.uk/srs5bin/cgi-bin/wgetz?-e+[PROSITEDOC-ID:$id]',
				'SRS-Japan http://srs.dna.affrc.go.jp/srs//srsc?[PROSITEDOC-id:$id]',
				'SRS-USA http://iubio.bio.indiana.edu:81/srs//srsc?[PROSITEDOC-id:$id]' 
				],
		 'MIM' => [ 'SRS-UK http://www.sanger.ac.uk/srs5bin/cgi-bin/wgetz?-e+[OMIM-ID:$id]',
			],
		 'PROSITE_PROFILE' => [ 'ISREC http://ulrec3.unil.ch/cgi-bin/get_pstprf?$id' ,
				    ],
		 'SCOP' => [ 'SCOP-UK http://scop.mrc-lmb.cam.ac.uk/scop/search.cgi?pdb=$id&tlev=$add',
			     'SCOP-USA http://scop.stanford.edu/scop/search.cgi?pdb=$id&tlev=$add',
			     'SCOP-Japan http://www.beri.co.jp/scop/search.cgi?pdb=$id&tlev=$add',
			     'CATH-PDBSUM http://www.biochem.ucl.ac.uk/bsm/pdbsum/$id/main.html'
			 ],
		 'SMART' => [ 'EMBL http://www.bork.embl-heidelberg.de/modul/do_annotation.pl?NAME=$id&BLAST=DUMMY'],
		 'PRINTS' => [ 'UCL http://www.biochem.ucl.ac.uk/cgi-bin/attwood/DoPRINTS.pl?cmd_a=Display&qua_a=none&fun_a=Text&qst_a=$id' ],
                 'PRODOM' => [ 'PRODOM http://www.toulouse.inra.fr/prodom/cgi-bin/ReqProdomII.pl?id_dom1=$id',
                              'SRS-UK http://www.sanger.ac.uk/srs5bin/cgi-bin/wgetz?-e+[PRODOM-PAC:$id]' ],
		 'PFAMB' => [ 'CGR http://www.cgr.ki.se/cgi-bin/Pfam/pfambget.pl?name=$id'],
		 'PDB' => [ 'Rasmol http://www.cgr.ki.se/cgi-bin/Pfam/pdb2rasmol.pl?pdb=$pdbid&chain=$pdbchain&start=$pdbstart&stop=$pdbstop'],
		 'MEROPS' => [ 'MEROPS http://www.bi.bbsrc.ac.uk/Merops/famcards/$id.htm'],
		 'CAZY' => [ 'CAZy http://afmb.cnrs-mrs.fr/~pedro/CAZY/$id.html'],
		 'EC' => [ 'Expasy http://www.expasy.ch/cgi-bin/nicezyme.pl?$id'],
		 'INTERPRO' => [ 'Interpro http://interpro.ebi.ac.uk/servlet/IEntry?ac=$id']
		 );
	
#
# This is the routine you need to change if you
# want to change how the alignment files around
# found
#

sub get_alignment {
    my $acc = shift;
    my $type = shift; # either 'full' or 'seed'
    my $is_unheaded =shift;
    my $ali;


    $ali  = new Align;
    
    open(ALIGN,"gunzip -c $data_root/$type/$acc.full.gz |") || do {
	if( $is_unheaded ) {
	    print "Content-type: text/html\n\n";
	}
	&PfamWWWConfig::exit_html("failed to open file [$data_root/$type/$acc.full] as an alignment file $!");
    };

    if( $ali->read_Pfam(\*ALIGN) == 0 ) {
	if( $is_unheaded ) {
	    print "Content-type: text/html\n\n";
	}
	&PfamWWWConfig::exit_html("Unable to read any alignment for $acc");
    }

    close(ALIGN);

    return $ali;
}

#
# This is the routine to change to
# get the description files
#

sub desc_FH_by_acc {
    my $acc = shift;
    my $fh;

    $fh  = new FileHandle;
    	
    $fh->open("$desc_dir/$acc.desc");
    return $fh;
}
    

#
# This is the routine to change to get to
# a protein entry from an ID. NB - this should 
# on the pfamseq database to allow all ids to be
# seen
#

sub protein_FH_by_name {
    my $name = shift;
    my $fh;

    $fh  = new FileHandle;

    $fh->open("$file_root/efetch PQ:$name |");

    return $fh;
}

#
# This is the routine to change to get to
# a protein entry from an accession. NB - this should 
# on the pfamseq database to allow all ids to be
# seen
#

sub protein_FH_by_acc {
    my $name = shift;
    my $fh;

    $fh  = new FileHandle;

    $fh->open("$file_root/efetch PQ:$name |");

    return $fh;
}

#
# This is the routine to change to get to
# swisspfam entry from the ID
#
#

sub swisspfam_FH_by_name {
    my $name = shift;
    my $fh;

    $fh = new FileHandle;

    $fh->open("$file_root/efetch pfam:$name |");

    return $fh;
}
    
#
# This is the routine to get the pfamb entry 
# from a PfamB name
#

sub pfamb_FH_by_name {
    my $name = shift;
    my $fh;

    $fh = new FileHandle;

    $fh->open("$file_root/efetch pfam:$name |");

    return $fh;
}

#
# This is the routine to get the pfama entry 
# from a PfamA name
#

sub pfama_FH_by_name {
    my $name = shift;
    my $fh;

    $fh = new FileHandle;

    $fh->open("$file_root/efetch pfam:$name |");

    return $fh;
}

#
# This routine to get a Pfam HMM from its accession
#

sub pfamhmm_by_name {
    my $name = shift;
    my $db = shift;
    my $fh;

    if ($db eq 'frag') {
       $db = 'PfamFrag';
    }
    else {
       $db = 'Pfam';
    }    

    $fh = new FileHandle;

    $fh->open("$file_root/hmmfetch $data_root/$db $name |");

    return $fh;
}

#
# This routine is called when there is in an error in any script
#
#

sub exit_html {
    my $mess = shift;
    my $noheader = shift;

    if( defined $noheader ) {
	print "Content-type: text/html\n\n";
    }

    print "<p>Sorry - the script failed due to errors. [$mess]. Please contact $contact if you feel there is a problem<p>\n";
    return 0;
}

#
# This routine is for the headers
#
#

sub header {
    my $title = shift;
    my ($varhash,$outstr);
    
    #pick up the header file, set up the variables
    
    
    $varhash{'title'} = $title;
    $varhash{'root'}  = $local_root;
    $varhash{'site'}  = $site;
    $varhash{'email'} = $email;
    $varhash{'admin'} = $adminname;

    open(HEAD,$header_file) || die "Could not open $header_file $!";
    while(<HEAD>) {
	while( $_ =~ /\$(\w+)/g ) {
	    
	    $var = $1;
	    # print STDERR "Got $var as a subs [$varhash{$var}]\n";
	    $temp = $varhash{$var};
	    s/\$$var/$temp/; # here it is...
	}
	$outstr .= $_;
    }
    close(HEAD);

    return $outstr;
}

# 
# This is for the footers
#
# 

sub footer {
    my $title = shift;

    # ignore the title!

    return "<hr noshade size=4>If you think there is anything wrong with this script, please contact $contact\n";
}

#
# Makes a sequence object.
#

sub seq_from_lines {
    my @lines = @_;
    my ($seq,$line,$name,$seqc);

    $seq = new Sequence;


    # find the first blank line. Probably a neater way to do this in perl
    #for($line = shift @lines;1;$line = shift @lines ) {
    #	if( $line =~ /\S/ ) {
    #	    last;
    #	}
    #}
    for(; $line = shift @lines;) {
	if( $line =~ /\S/ ) {
	    last;
	}
    }
	
    if( ($line =~ /^\s*\>\s*(\w*)/) ) {
	$name = $1;
	$seqc = "";
    } else {
	$seqc = $line;
    }

    if( $name =~ /^\s*$/ ) {
	$name = "NoName";
    }

    $seqc .= join("",@lines);

    
    $seqc =~ tr/[a-z]/[A-Z]/;
    $seqc =~ s/[^A-Z]//g;

    $seq = new Sequence();

    $seq->name($name);
    $seq->seq($seqc);

    return $seq;
}

sub seq_from_lines_mats { # Mats' version
    my @lines = @_;
    my ($seq,$line,$name,$seqc);

    $seq = new Sequence;

    $line = shift @lines;
    if( ($line =~ /^\s*\>(\w+)/) ) {
	$name = $1;
	$seqc = "";
    } else {
	$seqc = $line;
    }

    $seqc .= join("",@lines);

    
    $seqc =~ tr/[a-z]/[A-Z]/;


    $seq = new Sequence();

    $seq->name($name);
    $seq->seq($seqc);

    return $seq;
}

sub seq_from_lines_ewan {
    my @lines = @_;
    my ($seq,$line,$name,$seqc);

    $seq = new Sequence;


    # find the first blank line. Probably a neater way to do this in perl
    for($line = shift @lines;1;$line = shift @lines ) {
	if( $line =~ /\S/ ) {
	    last;
	}
    }
	
    if( ($line =~ /^\s*\>\s*(\w*)/) ) {
	$name = $1;
	$seqc = "";
    } else {
	$seqc = $line;
    }

    $seqc .= join("",@lines);

    
    $seqc =~ tr/[a-z]/[A-Z]/;
    $seqc =~ s/[^A-Z]//g;

    $seq = new Sequence();

    $seq->name($name);
    $seq->seq($seqc);

    return $seq;
}

1;  # says use was ok
__END__

=head1 NAME

PfamWWWConfig

=head1 DESCRIPTION

Description for B<PfamWWWConfig>

=head1 AUTHOR

B<Ewan Birney> Email birney@sanger.ac.uk

=over
