
=head1 NAME

swiss2gif

=head1 DESCRIPTION

This is the package for making the gifs in Pfam.

It was written by Mats Johnson at the Karolinska Institutet
with Erik Sonnhammer. <esr@kisac.cgr.ki.se>

Ewan Birney <birney@sanger.ac.uk> is the closest person to the
code at the moment. The following changes were made from Mats
original code

 1) Removed the production of the <html> tags for each gif 

 2) Added a print_html function that can optionally taking a leading
 first argument. The print_html defaul to stdout if the first
 argument is undef, if not it uses $arg->print() method.
 (consistent in particular with the Paging object).

 This allows use to redirect the gif production elsewhere if we
 want to.

 3) fixed a javascript bug for onMouseOver statements


Problems with this module

    a) This module is too tightly coupled with the PfamWWWConfig module
       to allow its use elsewhere

    b) The gif production is a function which both parses swisspfam format
       and produces the gifs, tieing it to swisspfam format.
=cut



package swiss2gif;

use PfamWWWConfig;

# Color table
 $R[0]=0;
 $G[0]=3;
 $B[0]=192;

 $R[1]=31;
 $G[1]=192;
 $B[1]=31;

 $R[2]=192; 
 $G[2]=15;
 $B[2]=15;

 $R[3]=189;
 $G[3]=192;
 $B[3]=0;

 $R[4]=192;
 $G[4]=8;
 $B[4]=174;

 $R[5]=0;
 $G[5]=186;
 $B[5]=192;

 $R[6]=132;
 $G[6]=132;
 $B[6]=192;

 $R[7]=147;
 $G[7]=192;
 $B[7]=144;

 $R[8]=192;
 $G[8]=175;
 $B[8]=146;

 $R[9]=133;
 $G[9]=133;
 $B[9]=230;

 $R[10]=142;
 $G[10]=37;
 $B[10]=17;

 $R[11]=242;
 $G[11]=146;
 $B[11]=66;

 $R[12]=0;
 $G[12]=135;
 $B[12]=0;

 $R[13]=255;
 $G[13]=135;
 $B[13]=250;

 $R[14]=235;
 $G[14]=235;
 $B[14]=48;

 $R[15]=0;
 $G[15]=100;
 $B[15]=244;

 $R[16]=69;
 $G[16]=69;
 $B[16]=69;

 $R[17]=255;
 $G[17]=135;
 $B[17]=164;

sub print_html (@) {
    my $paging = shift;
    my @lines = @_;

    $line = join('',@lines);

#    if( defined $paging ) {
#	$paging->print($line);
#    } else 
    {
	print STDOUT "$line";
    }
}


sub domain_view {

my $ENTRY=shift; # In 'swisspfam' format
my $text_switch=shift; # 0 -> Brief format - protein name only (overview).
                       # 1 -> Show domain names and overlap icons (swisspfam/HMMsearch).
                       # 2 -> Show protein name, organism, and DESC (overview).
my $table_switch=shift;
local *color_tableA=shift;
local *color_tableB=shift;
my $paging = shift; # if we are using a paging module
my $zoom_factor = shift;

my $number_of_domains; # Total number of domains, including those of same sort
my $number_of_lines; # Number of lines in file
my $counter;
my $outer_counter;
my $color_counterA;
my $color_counterB;

my $protein_length;  # Length of protein  
my @no_same_domain;  # Number of 'repeats' of a certain domain
my @PfamB_flag;      # Set to 1 if belongs to Pfam-B, 0 if Pfam-A
my $number_of_overlaps;

$number_of_domains=0;
$number_of_lines=0;
$outer_counter=0;
$number_of_overlaps=0;


 # 2 = Exclusive lock
flock ($ENTRY,2);

while (<$ENTRY>)   # Read file to line array
{
 $line[$number_of_lines] = $_;
 $number_of_lines++;
}

# 8 = Unlock
flock ($ENTRY,8);
close ($ENTRY);

# Read protein length and name
($protein_length) = ($line[0]=~/(\d+)\sa\.a\./);
($protein_name) = ($line[0]=~/>([^\s]*)/); 
#print STDOUT "Ok... the name here is [$protein_name]<p>\n";

# Read number of 'repeats' for each domain
for ($counter=1;$counter<$number_of_lines;$counter++) 
{
  ($no_same_domain[$counter])=($line[$counter]=~/\s+(\d+)/);
}




# Read start and end, name and accession number for each domain
# Repeated domains are stored succesively
for ($counter=1;$counter<$number_of_lines;$counter++)
{
 $dummyline=$line[$counter];
 for ($loop=0; $loop < $no_same_domain[$counter]; $loop++)
   {
      $number_of_domains++;
      ($end[$number_of_domains-1])=($dummyline=~/(\d+)\s?$/); 
      ($beg[$number_of_domains-1])=($dummyline=~/(\d+)-\d+\s?$/);
      ($dummyline,$waste)=split(/\d+-\d+\s?$/,$dummyline); 
       $dummyline=$dummyline . "\n";
      ($domain_name[$outer_counter])=($dummyline=~/([^\s]*)/);
      ($domain_acc[$outer_counter])=($dummyline=~/(P[BF]\d+)/);
      $outer_counter++;
    }  
   ($description_dummy[$counter])=($dummyline =~ /PF\d+([^\n]*)/);
}

# Line up descriptions and pfam-B names for display in ALT text box

$outer_counter=0;
for ($counter=1;$counter<$number_of_lines;$counter++)
{
    for ($loop=0;$loop<$no_same_domain[$counter];$loop++)
    {
     if ($domain_name[$outer_counter]=~/Pfam-B/)
     {
      $text_box[$outer_counter]=$domain_name[$outer_counter];
      $outer_counter++; 
     }
     else
     {
      $text_box[$outer_counter]= $description_dummy[$counter];
      chop($text_box[$outer_counter]); # Remove trailing spaces
      chop($text_box[$outer_counter]);
      $outer_counter++; 
     }
    }
  
}



# Keep track of used colors

$color_counterA=@color_tableA; # How many colors have been used?
$color_counterB=@color_tableB;

if ($color_counterA==0)
{
    $color_counterA=1;
}

for ($counter=0;$counter<$number_of_domains;$counter++)
{  
    if ($domain_acc[$counter]=~/PF\d\d\d\d\d/) # Pfam-A
    { 
	$PfamB_flag[$counter]=0;
	$match=0;
	for ($loop=0;$loop<$color_counterA;$loop++)
	{
	    if ($domain_acc[$counter] eq $color_tableA[$loop])
	    {
		$match=1;
		$old_imageA=$loop; # Remember this value
	    }
	}
	if ($match==1) # So, it was previously used
	{ 
	    $image_name[$counter]=$PfamWWWConfig::imageA . $old_imageA . ".gif";
	}
	else # Not previously used, assign new color 
	{
	    $image_name[$counter]=$PfamWWWConfig::imageA . $color_counterA . ".gif";
	    $color_tableA[$color_counterA]=$domain_acc[$counter];
	    $color_counterA++;
	}
	$my_file=$PfamWWWConfig::tempdir . "/" . $image_name[$counter];
	if (!(-e $my_file)) # Do we have this gif already?
	{
	    generate_2stripedGIF($color_counterA-1); 
	}
	$image_name[$counter]=$PfamWWWConfig::image_temp . "/" . $image_name[$counter] ;
	
    } 
    else  # i.e pfamB
    {
	$PfamB_flag[$counter]=1; 
	$match=0;
	for ($loop=0;$loop<$color_counterB;$loop++) # Same test as above
	{
	    if ($domain_acc[$counter] eq $color_tableB[$loop])
	    {
		$match=1;
		$old_imageB=$loop;
	    }
	}
	if ($match==1)
	{ 
	    $image_name[$counter]=$PfamWWWConfig::imageB . $old_imageB . ".gif";
	    $PfamB_flag[$counter]=1;
	}
	else
	{
	    $image_name[$counter]=$PfamWWWConfig::imageB . $color_counterB . ".gif";
	    $color_tableB[$color_counterB]=$domain_acc[$counter];
	    $color_counterB++;
	}

	$my_file=$PfamWWWConfig::tempdir . "/" . $image_name[$counter];
	
	if (!(-e $my_file))
	{
	    generate_3stripedGIF($color_counterB-1);
	}
	$image_name[$counter]=$PfamWWWConfig::image_temp . "/" . $image_name[$counter] ;
    }
    
    $dont_draw[$counter]=0; # Just use loop to reset every $dont_draw

}


# Sort domains in order of appearance

$flag=1;
while($flag==1) { 
    $flag=0;
    for ($counter=1; $counter<$number_of_domains; $counter++) 
    {
	if ($beg[$counter-1] > $beg[$counter])
	{
	    $flag=1;
	    $beg_swap=$beg[$counter];
	    $end_swap=$end[$counter];
	    $domain_swap=$domain_name[$counter];
	    $acc_swap=$domain_acc[$counter];
	    $text_swap=$text_box[$counter];
	    $image_swap=$image_name[$counter];
	    $PfamB_swap=$PfamB_flag[$counter];

	    $beg[$counter]=$beg[$counter-1];
	    $end[$counter]=$end[$counter-1];
	    $domain_name[$counter]=$domain_name[$counter-1];
	    $domain_acc[$counter]=$domain_acc[$counter-1];
	    $text_box[$counter]=$text_box[$counter-1];
	    $image_name[$counter]=$image_name[$counter-1];
	    $PfamB_flag[$counter]=$PfamB_flag[$counter-1];

	    $beg[$counter-1]=$beg_swap;
	    $end[$counter-1]=$end_swap;
	    $domain_name[$counter-1]=$domain_swap;
	    $domain_acc[$counter-1]=$acc_swap;
	    $text_box[$counter-1]=$text_swap;
	    $image_name[$counter-1]=$image_swap;
	    $PfamB_flag[$counter-1]=$PfamB_swap;
	}
    }
}

$space_name=$PfamWWWConfig::space;
# $line_name=$PfamWWWConfig::line;
# $image_name=$PfamWWWConfig::imageA;
$overlap_name=$PfamWWWConfig::overlap;
$blank_name=$PfamWWWConfig::blank;

$overlap_picture_width=59;
$overlap_picture_height=30;


# Set length of figure on screen
# A zoom factor of 1 => one a.a. per pixel
# [zoom factor is now set interactively] $zoom_factor=0.5;  # 2 a.a. per pixel

# Make copy of beg/end that we can mess with
for ($loop=0;$loop<$number_of_domains;$loop++)
{
    $screen_beg[$loop]=$beg[$loop];
    $screen_end[$loop]=$end[$loop];
}


# Take care of overlapping domains so that they are drawn correctly.
# The idea is to center the overlap icon over the overlap.

for ($i=0; $i < $number_of_domains-1; $i++)
{
    for ($j=$i+1; $j < $number_of_domains; $j++)
    {
        if ($i != $j && $screen_end[$i] > $screen_beg[$j])
        {
  	    if ($screen_end[$i] < $screen_end[$j])     # Like this:  i  --------
	    {                                          #             j       -------

	        $screen_overlap[$number_of_overlaps]=($screen_end[$i]-$screen_beg[$j])/2+$screen_beg[$j];
	        # $screen_overlap[] is the central position of the overlap icon.

	        # Remove overlapping bit of Pfam-B segment (assume Pfam-A segments don't overlap)
	        if ($PfamB_flag[$j]==0)
	        {
		    $screen_end[$i]=$screen_beg[$j]-1;
		}
	        else
	        {
		    $screen_beg[$j]=$screen_end[$i]+1;
		}
	    }
	    else                                           # Like this:  i  -------- 
	    {                                              #             j     ---
	      
	      $screen_overlap[$number_of_overlaps]=($screen_end[$j]-$screen_beg[$j])/2+$screen_beg[$j];
	      
	      if ($PfamB_flag[$i] && !$PfamB_flag[$j]) {
		
		# j is Pfam-A domain - chop domain i up into two overhanging bits.
		
		# Insert new domain of right overhang
		for ($k = $number_of_domains; $k > $j+1; $k--) {
		  $beg[$k]=$beg[$k-1];
		  $end[$k]=$end[$k-1];
		  $screen_beg[$k]=$screen_beg[$k-1];
		  $screen_end[$k]=$screen_end[$k-1];
		  $domain_name[$k]=$domain_name[$k-1];
		  $domain_acc[$k]=$domain_acc[$k-1];
		  $text_box[$k]=$text_box[$k-1];
		  $image_name[$k]=$image_name[$k-1];
		  $PfamB_flag[$k]=$PfamB_flag[$k-1];
		  $dont_draw[$k]=$dont_draw[$k-1];
		}
		$number_of_domains++;
		$beg[$k]=$beg[$i];
		$end[$k]=$end[$i];
		$screen_beg[$k]=$screen_end[$j]+1;
		$screen_end[$k]=$screen_end[$i];
		$domain_name[$k]=$domain_name[$i];
		$domain_acc[$k]=$domain_acc[$i];
		$text_box[$k]=$text_box[$i];
		$image_name[$k]=$image_name[$i];
		$PfamB_flag[$k]=$PfamB_flag[$i];

                # Truncate domain i
		$screen_end[$i]=$screen_beg[$j]-1;
	      }
	      else {
		$dont_draw[$j]=1;
	      }
	    }
	 
    	    $number_of_overlaps++;
	}
    } 

}

# Actual output

if ($text_switch==1)
{
    print_html $paging, "<HTML> \n <HEAD><TITLE>swiss2gif</TITLE></HEAD> \n";
    print_html $paging, "<BODY BGCOLOR=#FFFFFF>";
    print_html $paging, "<BR>";

    if ($number_of_overlaps>0)
    {
        my $i;
        my $curpos = 0;

	print_html $paging,"<br>\n";

	for($i=0;$i<$number_of_overlaps;$i++)
	{
	    print_html $paging,"<IMG SRC=",$blank_name," HSPACE=0 ALIGN=LEFT BORDER=0 WIDTH=",($screen_overlap[$i] - $curpos)*$zoom_factor - $overlap_picture_width/2,"  HEIGHT=10 > \n";
	    print_html $paging,"<IMG SRC=",$overlap_name," HSPACE=0 ALIGN=LEFT BORDER=0 WIDTH=",$overlap_picture_width," HEIGHT=",$overlap_picture_height," >\n";
            $curpos = $screen_overlap[$i]+$overlap_picture_width/2;
	}
	print_html $paging,"<br>\n";
    }

}



print_html $paging,"<BR>\n";

if ($text_switch==0)
{
    # Make each name equally long so we can put name and schematic on one line.
    # However, the GIFs don't work within <PRE>, and the correct spacing needs <PRE>.
    # Maybe a table would work better?
     my $i, $len=length($protein_name);
     for ($i=0; $i < 10-$len; $i++) { $protein_name = $protein_name." " };

#    print_html $paging,"<PRE>";
    print_html $paging,"<A HREF=$PfamWWWConfig::swisspfam?name=",$protein_name,">",$protein_name,"</A>";
    print_html $paging,"<p>"; # Without this the GIFs are incomplete (!)
}

if ($text_switch==2)
{
#    print_html $paging,"<PRE>";
    print_html $paging,"<A HREF=$PfamWWWConfig::swisspfam?name=",$protein_name,">",$protein_name,"</A> ";

    # Print organism
    $entry = &PfamWWWConfig::protein_FH_by_name($protein_name);
    if( ! defined $entry ) {
      &PfamWWWConfig::exit_html("Unable to get the entry from $name");
    }
    my $s = "[";
    while ( <$entry> ) {
      if( /^OS/ ) {
	s/^OS\s+//g;
	s/\.\n//g;
	$_ = lc;
	$_ = ucfirst;
	$s = $s.$_;
      }
    }
    close($entry);
    $s = $s."\] ";
    print $paging, $s;

    # Print description
    $entry = &PfamWWWConfig::protein_FH_by_name($protein_name);
    if( ! defined $entry ) {
      &PfamWWWConfig::exit_html("Unable to get the entry from $name");
    }
    while ( <$entry> ) {
      if( /^DE/ ) {
	s/^DE\s+//g;
	s/\.\n//g;
	print $paging, lc $_;
      }
    }
    close($entry);

    print_html $paging,"<p>"; # Without this the GIFs are incomplete (!)
}

$scheme_height=23;  # Sets height of figure

# Initial spacer
if ($screen_beg[0]>1)
{
    print_html $paging,"<IMG SRC=",$space_name," HSPACE=0 ALIGN=LEFT BORDER=0 HEIGHT=",$scheme_height," WIDTH=",$screen_beg[0]*$zoom_factor,">\n";
}

for ($counter=0; $counter < $number_of_domains; $counter++)
{
    if (!$dont_draw[$counter]) # Don't draw segments hidden because of overlap
    {
	if ($PfamB_flag[$counter]==0) # Pfam-A
	{
	    # subs out back-ticks before putting into javascript (Ewan)
	    $temp_text=$text_box[$counter];
	    $temp_text=~ s/\'/\\\'/g;

	    print_html $paging,"<A HREF=$PfamWWWConfig::getacc?",$domain_acc[$counter]," onMouseOver=\"window.status='",$temp_text,"';return true\" onMouseOut=\"window.status=''\";return true; > <IMG SRC=",$image_name[$counter]," HSPACE=0 ALIGN=LEFT BORDER=0 HEIGHT=",$scheme_height," WIDTH=",($screen_end[$counter]-$screen_beg[$counter])*$zoom_factor, " ALT=\"",$text_box[$counter],"\"></A>\n";
	}
	else # Pfam-B
	{
	    # subs out back-ticks before putting into javascript (Ewan)
	    # not really necessary as it is Pfam-B... but you never know!
	    $temp_text=$text_box[$counter];
	    $temp_text=~ s/\'/\\\'/g;

	    print_html $paging,"<A HREF=pfambget.pl?name=",$domain_name[$counter]," onMouseOver=\"window.status='",$temp_text,"';return true\" onMouseOut=\"window.status=''\";return true; > <IMG SRC=",$image_name[$counter]," HSPACE=0 ALIGN=LEFT BORDER=0 HEIGHT=",$scheme_height," WIDTH=",($screen_end[$counter]-$screen_beg[$counter])*$zoom_factor, " ALT=\"",$text_box[$counter],"\" ></A>\n";
	}

        # Put spacer inbetween
	if (($counter < $number_of_domains-1)) {

          # Find distance to next visible segment
	  $found = 0;
	  
	  $k = $counter+1;
	  while ($k < $number_of_domains && !$found) {
	    if (!$dont_draw[$k]) { 
	      $found = 1;
	      $dist = $screen_beg[$k] - $screen_end[$counter]; 
	      #print $counter, " ", $dist, "\n";
	    }
	    $k++;
	  }
	  if ($dist > 0) {
	    print_html $paging," <IMG SRC=",$space_name," ALIGN=LEFT HSPACE=0 BORDER=0 HEIGHT=",$scheme_height," WIDTH=",$dist*$zoom_factor, ">\n ";
	  }
	}
    } # End if $dont_draw ==0
} # End for $counter...


# final spacer
if ( ($protein_length - $screen_end[$number_of_domains-1]) > 0 )
{
    print_html $paging," <IMG SRC=",$space_name," ALIGN=LEFT HSPACE=0 BORDER=0 HEIGHT=",$scheme_height," WIDTH=",($protein_length - $screen_end[$number_of_domains-1])*$zoom_factor, ">\n "; 
}

# Add some space at the end to always fit domain text into window
print_html $paging,"<IMG SRC=",$blank_name," ALIGN=LEFT HSPACE=0 BORDER=0 HEIGHT=2 WIDTH=100>\n<BR>";

if ($text_switch==1)  # Print domain names and position beneath figure
{
    print_html $paging,"<BR>";
    for ($loop=0;$loop<$number_of_domains;$loop++) {
        if ($PfamB_flag[$loop]==0 && !$dont_draw[$loop])
        {
	    print_html $paging,"<IMG SRC=",$blank_name," HSPACE=0 ALIGN=LEFT BORDER=0 WIDTH=",($screen_beg[$loop])*$zoom_factor," HEIGHT=2 >"; 
	    print_html $paging,"<B>",$domain_name[$loop],"&nbsp</B>",$screen_beg[$loop],"-",$screen_end[$loop];
	    print_html $paging,"\n<BR>\n";
        }
    }
}

print_html $paging,"<P>";

if ($table_switch==1) # Print table
{
    print_html $paging,"<P><CENTER><TABLE BORDER><TR><TH>Domain</TH><TH>Start</TH><TH>End</TH>\n";

    for ($loop=0;$loop<$number_of_domains;$loop++)
    {
	if ($PfamB_flag[$loop]==0) # Pfam-A
	{
	    print_html $paging,"<TR><TD><A HREF=$PfamWWWConfig::getacc?",$domain_acc[$loop],">",$domain_name[$loop],"</A></TD><TD>",$beg[$loop],"</TD><TD>",$end[$loop];
	}
	else # Pfam-B
	{
	    print_html $paging,"<TR><TD><A HREF=pfambget.pl?name=",$domain_name[$loop],">",$domain_name[$loop],"</A></TD><TD>",$beg[$loop],"</TD><TD>",$end[$loop];
	}
    }

    print_html $paging,"</TABLE></CENTER><BR><BR>";
}


} # End sub



sub generate_3stripedGIF {

    my $n=shift;
    use integer;

    my $i1;
    my $i2;
    my $i3;

    $gif_image="$PfamWWWConfig::image_dir/3bands_small.gif";  # Template image
    $dest_gif_image="$PfamWWWConfig::tempdir/colors$n.gif";
    if (open (IMAGE, "<" . $gif_image)) # Open template image
{
    $no_bytes=(stat ($gif_image))[7];
    $piece_size=1;

    # Color-spreading algorithm

    $x=(3*$n)%18 + ($n/6)%3; # 'Window' 
    $i1=($x)%18;
    $i2=($x+1+$n/324)%18;
    $i3=($x+2+$n/18)%18;

    if (open (DEST_IMAGE,">" . $dest_gif_image)) # Open output image
    {
	binmode(DEST_IMAGE);
	for ($loop=0;$loop<=$no_bytes;$loop+=$piece_size)
	{
	    read (IMAGE,$data,$piece_size);

	    #$i1,$i2 and $i3 should contain the color number for band 1, 2 and 3.
	    
	    if ($loop==13) # Start of color information
	    {
		#Low-level IO
		
		$data=chr(&control_limits($R[$i3]-50));  #Byte 13,14,15 (counting from zero) Dark shade
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($G[$i3]-50)); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($B[$i3]-50)); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr(&control_limits($R[$i3]-20));  #Byte 16,17,18 Lighter shade
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($G[$i3]-20)); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($B[$i3]-20)); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr(&control_limits($R[$i1]+20));  #Byte 19,20,21 Darker reflection
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($G[$i1]+20)); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($B[$i1]+20)); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr($R[$i1]);                      #Byte 22,23,24 First band           
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($G[$i1]); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($B[$i1]); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr($R[$i2]);                      #Byte 25,26,27 Second band
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($G[$i2]); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($B[$i2]); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr(&control_limits($R[$i1]+40));  #Byte 28,29,30 Bright reflection
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($G[$i1]+40)); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($B[$i1]+40)); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr($R[$i3]);                      #Byte 31,32,33 Third band
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($G[$i3]); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($B[$i3]); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$loop=33;

		for ($count=13;$count<33;$count+=$piece_size)  #Catch up with destination image
		{
		    read (IMAGE,$dummy,$piece_size);
		}


	    } 
	    
	    if ($loop!=33) #Have already written this byte
	    { 
		syswrite(DEST_IMAGE,$data,$piece_size);
	    }

	} # End loop

	close (DEST_IMAGE); 
    }
    else
    {
	print "Content-type: text/plain","\n\n";
	print "Could not write to output file.";
    }
    close (IMAGE);
}
else
{
    print "Content-type: text/plain","\n\n";
    print "Could not open template image.";
}

} # End sub




sub generate_2stripedGIF {

    my $n=shift;
    use integer;

    $gif_image="$PfamWWWConfig::image_dir/2bands.gif";   # Template image
    $dest_gif_image="$PfamWWWConfig::tempdir/colorsA$n.gif";
    if (open (IMAGE, "<" . $gif_image))
{
    $no_bytes=(stat ($gif_image))[7];
    $piece_size=1;

    if ($n<18)  # One-colored as long as possible
    {
	$i1=$n;  #Same color
	$i2=$n;
    }
    if ($n>17 && $n<99)
    {
	$i1 = $n % 9;     #All combinations of first nine colors on upper stripe
	$i2 = 7+($n / 9); #and last nine colors on lower stripe.
    }
    if ($n>98 && $n<181)
    {
	$i1 = ($n / 9)-2; #All combinations of last nine colors on upper stripe
	$i2 = $n % 9;     #and first nine colors on lower stripe.
    }
    # A total of 18+81+81=180 combinations should be enough
    # for all Pfam-A:s in proteins sharing at least one domain.
    

    if (open (DEST_IMAGE,">" . $dest_gif_image))
    {
	binmode(DEST_IMAGE);
	for ($loop=0;$loop<=$no_bytes;$loop+=$piece_size)
	{
	    read (IMAGE,$data,$piece_size);

	    #$i1 and $i2 should contain the color number for stripe 1 and 2.
	    
	    if ($loop==13) # Start of color information
	    {
		#Low-level IO
		
		$data=chr(&control_limits($R[$i1]+20));  #Byte 13,14,15 Darker reflection (counting from 0)
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($G[$i1]+20)); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($B[$i1]+20)); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr(&control_limits($R[$i2]-50));  #Byte 16,17,18 Dark shade
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($G[$i2]-50)); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($B[$i2]-50)); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr($R[$i1]);                      #19,20,21 First band           
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($G[$i1]); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($B[$i1]); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr(&control_limits($R[$i2]-20));  #Byte 22,23,24 Light shade
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($G[$i2]-20)); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($B[$i2]-20)); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr($R[$i2]);  #25,26,27 Second band
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($G[$i2]); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr($B[$i2]); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$data=chr(&control_limits($R[$i1]+40));  #Byte 28,29,30 Bright reflection
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($G[$i1]+40)); 
		syswrite(DEST_IMAGE,$data,$piece_size);
		$data=chr(&control_limits($B[$i1]+40)); 
		syswrite(DEST_IMAGE,$data,$piece_size);

		$loop=30;

		for ($count=13;$count<30;$count+=$piece_size)  #Catch up with destination image
		{
		    read (IMAGE,$dummy,$piece_size);
		}

	    } 
	    
	    if ($loop!=30) #Have already written this byte
	    { 
		syswrite(DEST_IMAGE,$data,$piece_size);
	    }

	}

	close (DEST_IMAGE);
    }
    else
    {
	print "Content-type: text/plain","\n\n";
	print "Could not write to output file.";
    }
    close (IMAGE);
}
else
{
    print "Content-type: text/plain","\n\n";
    print "Could not open template image.";
}

} # End sub



sub control_limits {

my $in=shift;
my $out;

$out=$in;
if ($in>255)
{
$out=255;
}

if ($in<0)
{
$out=0;
}

return $out;
}


1;  # All OK

__END__

=head1 NAME

swiss2gif

=head1 DESCRIPTION

Description for B<swiss2gif>

=head1 AUTHOR

Mats Jonsson

=over
